/* XmGoBang: (c) 1995-2008 by Andreas S. Krebs */

#include "xmgobang.h"

/* static typedefs, only needed here */

typedef struct
{
  String playerType;		/* contains computer or human */
  String playerName;
  Pixel  playerTopColor;
  Pixel  playerColor;
  Pixel  playerBottomColor;
} PlayerRes;

typedef struct
{
  PlayerRes 	Players[2];
  Pixel		boardTopColor;
  Pixel		boardColor;
  Pixel		boardBottomColor;
  Pixel 	boardBackground;
  Pixel 	boardBorderColor;
  Pixel		hintTopColor;
  Pixel		hintColor;
  Pixel		hintBottomColor;
  short 	startPlayer;
  String 	sizeTxt;
  String	playMode;
  String	cheatMode;
  Boolean	soundOn;
} ApplResType;


/* some static prototypes */

static void getPlayerRes(GoPlayer *, PlayerRes *);
static short getPlayMode(String);    
static short getCheatMode(String);   

/* my own resources */

static XtResource resources[]=
{
  { "playerOneType", "PlayerOneType", XmRString, sizeof(String),
    XtOffset(ApplResType*, Players[0].playerType), XmRString, "human" },
  { "playerOneName", "PlayerOneName", XmRString, sizeof(String),
    XtOffset(ApplResType*, Players[0].playerName), XmRString, "You" },
  { "playerOneTopShadowColor", "PlayerOneTopShadowColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, Players[0].playerTopColor), XmRString, "#fa0000" },
  { "playerOneColor", "PlayerOneColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, Players[0].playerColor), XmRString, "#c80000" },
  { "playerOneBottomShadowColor", "PlayerOneBottomShadowColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, Players[0].playerBottomColor), XmRString, "#960000" },  
  
  { "playerTwoType", "PlayerTwoType", XmRString, sizeof(String),
    XtOffset(ApplResType*, Players[1].playerType), XmRString, "computer" },
  { "playerTwoName", "PlayerTwoName", XmRString, sizeof(String),
    XtOffset(ApplResType*, Players[1].playerName), XmRString, "Me" },
  { "playerTwoTopShadowColor", "PlayerTwoTopShadowColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, Players[1].playerTopColor), XmRString, "#0000fa" },
  { "playerTwoColor", "PlayerTwoColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, Players[1].playerColor), XmRString, "#0000c8" },
  { "playerTwoBottomShadowColor", "PlayerTwoBottomShadowColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, Players[1].playerBottomColor), XmRString, "#000096" },

  { "boardTopShadowColor", "BoardTopShadowColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, boardTopColor), XmRString, "#00a400" },
  { "boardColor", "BoardColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, boardColor), XmRString, "#007800" },
  { "boardBottomShadowColor", "BoardBottomShadowColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, boardBottomColor), XmRString, "#004c00" },
  { "boardBackground", "BoardBackground", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, boardBackground), XmRString, "black" },
  { "boardBorderColor", "BoardBorderColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, boardBorderColor), XmRString, "Darkgreen" },

  { "hintTopShadowColor", "HintTopShadowColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, hintTopColor), XmRString, "#a4a400" },
  { "hintColor", "HintColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, hintColor), XmRString, "#787800" },
  { "hintBottomShadowColor", "HintBottomShadowColor", XmRPixel, sizeof(Pixel),
    XtOffset(ApplResType*, hintBottomColor), XmRString, "#4c4c00" },
  
  { "startPlayer", "StartPlayer", XmRShort, sizeof(short),
    XtOffset(ApplResType*, startPlayer), XmRString, "1" },
  { "boardSize", "BoardSize", XmRString, sizeof(String),
    XtOffset(ApplResType*, sizeTxt), XmRString, "20x15" },

  { "playMode", "PlayMode", XmRString, sizeof(String),
    XtOffset(ApplResType*, playMode), XmRString, "normal" },
  { "cheatMode", "CheatMode", XmRString, sizeof(String),
    XtOffset(ApplResType*, cheatMode), XmRString, "none" },
  
  { "soundOn", "SoundOn", XmRBoolean, sizeof(Boolean),
    XtOffset(ApplResType*, soundOn), XmRString, "False" }
};


/************************************************************************
 *									*
 * get the resources for XmGoBang				 	*
 *  									*
 ************************************************************************/

void getApplResources(Widget applShell)
{
  int  cl, rw;
  ApplResType applRes;
  
  XtGetApplicationResources(applShell, &applRes, resources, XtNumber(resources),
			    (Arg *)NULL, 0);
  
  getPlayerRes(&XGoGame.Player[0], &applRes.Players[0]);
  getPlayerRes(&XGoGame.Player[1], &applRes.Players[1]);

  XGoGame.StartPlayer=applRes.startPlayer -1;
  if(XGoGame.PlayersTurn < 0 || XGoGame.PlayersTurn > 1)
  {
    fprintf(stderr, "illegal start player: %d (allowed: 0 or 1)\n", applRes.startPlayer);
    XGoGame.StartPlayer=0;
  }
  
  sscanf(applRes.sizeTxt, "%ux%u", &cl, &rw);
  XGoGame.Columns=cl;
  XGoGame.Rows=rw;
  if(XGoGame.Columns < 5 || XGoGame.Columns > 50 || 
     XGoGame.Rows < 5 || XGoGame.Rows > 40)
  {
    fprintf(stderr, "illegal board size: %s (allowed: 5x5 to 50x40)\n");
    XGoGame.Columns=20;
    XGoGame.Rows=15;
  }

  XGoGame.BoardTopColor=applRes.boardTopColor;
  XGoGame.BoardColor=applRes.boardColor;
  XGoGame.BoardBottomColor=applRes.boardBottomColor;
  XGoGame.BoardBackground=applRes.boardBackground;
  XGoGame.BoardBorderColor=applRes.boardBorderColor;

  XGoGame.HintTopColor=applRes.hintTopColor;
  XGoGame.HintColor=applRes.hintColor;
  XGoGame.HintBottomColor=applRes.hintBottomColor;
  
  XGoGame.Width=(XGoGame.Columns-1)*FIELD_SIZE+2*BORDER_SIZE+IMAGE_SIZE+2;
  XGoGame.Height=(XGoGame.Rows-1)*FIELD_SIZE+2*BORDER_SIZE+IMAGE_SIZE+2; 

  XGoGame.PlayMode=getPlayMode(applRes.playMode);
  XGoGame.CheatMode=getCheatMode(applRes.cheatMode);  
  XGoGame.SoundOn=applRes.soundOn;
}


/************************************************************************
 *									*
 * get the player resources and set the GoPlayer structure		*
 *  									*
 ************************************************************************/

static void getPlayerRes(GoPlayer *player, PlayerRes *res)
{
  if(strcasecmp(res->playerType,"human")==0)
    player->Type=PLAYER_HUMAN;
  else if(strcasecmp(res->playerType,"computer")==0)
    player->Type=PLAYER_COMPUTER;
  else
  {
    fprintf(stderr,"illegal player type: %s (use human or computer)\n", res->playerType);
    player->Type=PLAYER_HUMAN;
  }
 
  player->Name=XtNewString(res->playerName);
  
  player->Seconds=0;
 
  player->TopColor=res->playerTopColor;
  player->Color=res->playerColor;
  player->BottomColor=res->playerBottomColor;
}


/************************************************************************
 *									*
 * get the play mode for xmgobang					*
 *  									*
 ************************************************************************/

static short getPlayMode(String str)
{
  if(strcasecmp(str,"normal")==0)
    return PLAY_NORMAL;
  else if(strcasecmp(str,"offensive")==0)
    return PLAY_OFFENSIVE;
  else if(strcasecmp(str,"defensive")==0)
    return PLAY_DEFENSIVE;
  else
  {
    fprintf(stderr,"illegal play mode: %s (use normal, offensive or defensive)\n", str);
    return PLAY_NORMAL;
  }
}


/************************************************************************
 *									*
 * get the cheat mode for xmgobang					*
 *  									*
 ************************************************************************/

static short getCheatMode(String str)
{
  if(strcasecmp(str,"none")==0)
    return CHEAT_NONE;
  else if(strcasecmp(str,"hint")==0)
    return CHEAT_HINT;
  else if(strcasecmp(str,"undo")==0)
    return CHEAT_UNDO;
  else
  {
    fprintf(stderr,"illegal cheat mode: %s (use none, hint or undo)\n", str);
    return CHEAT_NONE;
  }
}


