/* XmGoBang: (c) 1995-2008 by Andreas S. Krebs */

#include "xmgobang.h"

#ifdef SOUND_SUPPORT

#include <fcntl.h>
#define BUFFER_SIZE 32768	/* large enough */

/* some local prototypes */

static void startsound();

/* some local variables */

static int child_pid=0;
static int snd_pipes[2];



/************************************************************************
 *									*
 * Callback procedure for the `sound`-menu item:		 	*
 *  									*
 ************************************************************************/

void soundCB(Widget widget , XtPointer client_data, XtPointer call_data)
{
  XmToggleButtonCallbackStruct *togbut;
  
  togbut=(XmToggleButtonCallbackStruct *)call_data;
  
  XGoGame.SoundOn=togbut->set;
}
#endif


/************************************************************************
 *									*
 * This procedure creates another process with fork, if not already	*
 * done. The other process than plays the requested sound.		*
 *  									*
 ************************************************************************/

void playSound(String soundname)
{
#ifndef SOUND_SUPPORT
  return;
#else    
  if(XGoGame.SoundOn==False)
    return;
  
  if(child_pid==0)    /* no child created yet, so I do it now */
  {
    if(pipe((int *)&snd_pipes))
    {
      fprintf(stderr,"Unable to create pipes to communicate with child\n");
      return;
    }

    if((child_pid=fork())<0) 
    {
      fprintf(stderr,"Unable to create child prozess\n"); 
      return;
    }
        
    if(child_pid==0)	  	/* hey, I'm the child ;-) */
    {
      startsound(); 
          
      close(snd_pipes[0]); 	/* close the pipes ! */
      close(snd_pipes[1]);
          
      exit(GOOD);		/* child exits here */
    }       
  }
  else if(child_pid<0)  /* already tried to create child, but failed */
    return;
    
  /* child and pipes are ok */
  write(snd_pipes[1],soundname,256);	  /* tell child which sound to play */
#endif
}


#ifdef SOUND_SUPPORT

/************************************************************************
 *									*
 * This procedure is the child process, that waits for a soundfile	*
 * and plays it.							*
 *  									*
 ************************************************************************/

static void startsound()
{      
  char buf[BUFFER_SIZE]; 
  int aud_fd,in_fd;
  int cnt,err;
      
  for(;;)		/* loop, until child gets exit signal */
  {
    short p;
      
    /* using buf for file name is ok, cause it's only need it once ;-) */
    strcpy(buf,SOUNDS_DIR);
    strcat(buf,"/");	
    p=strlen(buf);
    read(snd_pipes[0],&buf[p], 256);
      
    if(!strcmp("EXIT",&buf[p]))
    return;	/* got exit message, so I exit ;-) */
        
    /* open audio device */
    if(!(aud_fd=open(AUDIO_DEV,O_WRONLY)))
    {	
      fprintf(stderr,"Unable to open audio device %s.\n",AUDIO_DEV);
      return;
    }
  
    /* open sound file */
    if((in_fd=open(buf,O_RDONLY,0))) 
    {
      /* copy data from sound file to audio device */
      while((cnt=read(in_fd,buf,BUFFER_SIZE)) > 0) 
      {
        err=write(aud_fd,buf,cnt);
	  
	if(err!=cnt) 
	{
	  fprintf(stderr, "Error, while writing to audio device\n");
	  break;
	}    
      }

      if(cnt<0) 
	fprintf(stderr, "Error, while reading sound file\n");

      /* Close the sound file */
      close(in_fd);      
    }
    else
      fprintf(stderr, "Unable to open sound file %s.\n",buf);
      	
    /* Close the audio device */
    close(aud_fd);
  }
}


/************************************************************************
 *									*
 * send sound child process the exit signal				*
 *  									*
 ************************************************************************/

void kill_soundchild()
{
  /* send EXIT to sound child, if it exists */
  if(child_pid>0)
    write(snd_pipes[1],"EXIT",256);	 
}
  
#endif


